/*
 *  * Copyright (C) 2002 Alejandro Aguilar Sierra <asierra@servidor.unam.mx>
 *  *
 *  * This program is free software; you can redistribute it and/or modify
 *  * it under the terms of the GNU General Public License as published by
 *  * the Free Software Foundation; either version 2 of the License, or
 *  * (at your option) any later version.
 *  *
 *  * This program is distributed in the hope that it will be useful,
 *  * but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  * GNU General Public License for more details.
 *  *
 *  * You should have received a copy of the GNU General Public License
 *  * along with this program; if not, write to the Free Software Foundation,
 *  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *  */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif


#include <gnome.h>

extern "C"
{
#include "interface.h"
#include "support.h"   
}

#include "dvtitler.h"

#include "ft_titler.h"


DVTitler::DVTitler()
{
   window = create_wintitler();  //create_window_mykinoplugin();
   text = 0;
   titler = 0;
   GnomeColorPicker *colorFG = GNOME_COLOR_PICKER(lookup_widget( GTK_WIDGET( window ), "colorpickerfg" ) );
   GnomeColorPicker *colorBG = GNOME_COLOR_PICKER(lookup_widget( GTK_WIDGET( window ), "colorpickerbg" ) );
   
   gnome_color_picker_set_i8(colorFG, 0xff, 0xff, 0xff, 0xff); 
   gnome_color_picker_set_i8(colorBG, 0, 0, 0, 0); 
}

DVTitler::~DVTitler()
{
   gtk_widget_destroy( window );
   delete titler;
}


char *DVTitler::GetDescription( ) const 
{
   return "DV Titler";
}


void DVTitler::AttachWidgets( GtkBin *bin )
{
   gtk_widget_reparent( ( GTK_BIN( window ) )->child, GTK_WIDGET( bin ) );
}

void DVTitler::DetachWidgets( GtkBin *bin )
{
   gtk_widget_reparent( ( GTK_BIN( bin ) )->child, GTK_WIDGET( window ) );
}

void DVTitler::InterpretWidgets( GtkBin *bin )
{
   GtkMenu *ihMenu = GTK_MENU( gtk_option_menu_get_menu( GTK_OPTION_MENU( lookup_widget( GTK_WIDGET( window ), "optionmenuih" ) ) ) ); 
   GtkMenu *ivMenu = GTK_MENU(gtk_option_menu_get_menu(GTK_OPTION_MENU(lookup_widget(GTK_WIDGET(window),"optionmenuiv" )))); 
   GtkMenu *fhMenu = GTK_MENU( gtk_option_menu_get_menu( GTK_OPTION_MENU( lookup_widget( GTK_WIDGET( window ), "optionmenufh" ) ) ) ); 
   GtkMenu *fvMenu = GTK_MENU(gtk_option_menu_get_menu(GTK_OPTION_MENU(lookup_widget(GTK_WIDGET(window),"optionmenufv" ))));   
   
   GtkEditable *titleEntry = GTK_EDITABLE( lookup_widget( GTK_WIDGET( window ), "text" ) );	
   GnomeFontPicker *fontPicker = GNOME_FONT_PICKER( lookup_widget( GTK_WIDGET( window ), "fontpicker" ) );
   GnomeColorPicker *colorFG = GNOME_COLOR_PICKER( lookup_widget( GTK_WIDGET( window ), "colorpickerfg" ) );
   GnomeColorPicker *colorBG = GNOME_COLOR_PICKER( 
		lookup_widget( GTK_WIDGET( window ), "colorpickerbg" ) );
  
   
   GtkWidget *active_item = gtk_menu_get_active( ihMenu );
   ih = g_list_index (GTK_MENU_SHELL (ihMenu)->children, active_item) ;

   active_item = gtk_menu_get_active( ivMenu );
   iv = g_list_index (GTK_MENU_SHELL (ivMenu)->children, active_item) ;
      
   active_item = gtk_menu_get_active( fhMenu );
   fh = g_list_index (GTK_MENU_SHELL (fhMenu)->children, active_item) ;

   active_item = gtk_menu_get_active( fvMenu );
   fv = g_list_index (GTK_MENU_SHELL (fvMenu)->children, active_item) ;

   if (fh > 2)
     fh = ih;
   
   if (fv > 2)
     fv = iv;
   
   pad = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(lookup_widget( GTK_WIDGET( window ), "spinpad")));
   
   // Obtain the text
   g_free( text );
   text = gtk_editable_get_chars( titleEntry, 0, -1 );

   // Obtain the font
   char *font = gnome_font_picker_get_font_name( fontPicker );
   
   titler = new FT_Titler(64, text, font);
   
   // Obtain the color
   gnome_color_picker_get_i8( colorFG, &colorfg.r, &colorfg.g, &colorfg.b, &colorfg.a);
   gnome_color_picker_get_i8( colorBG, &colorbg.r, &colorbg.g, &colorbg.b, &colorbg.a);
}

void DVTitler::FilterFrame( uint8_t *io, int width, int height, double position, double frame_delta )
{
   int w, h;   
   int x, y;
   float ph, pv;
   int row_w = width * 3;
   
//   FT_Titler titler(64, text);
   titler->getMetrics(w, h);
   
   ph = position*(fh - ih) + ih;
   pv = position*(fv - iv) + iv;
   x = (int)(ph*(width - w)/2 + (1 - ph)*pad);
   y = (int)(pv*(height - h)/2 + (1 - pv)*pad + h);
   titler->setPixels(io, row_w);
   titler->setColors(colorfg, colorbg);
   
   if (pad > 1 && colorbg.a > 50)
     drawRectangle(io, x-pad, y-h-pad, w+2*pad, h+2*pad, row_w);
   titler->drawText(x, y);
}

void DVTitler::drawRectangle( uint8_t *io, int x, int y, int w, int h, int row_w)
{
   float a = (float)colorbg.a/255.0;
   uint8_t *base = io + row_w *y + x*3;
   
   for (int j=0; j < h; j++) {
      uint8_t *p = base;
      for (int i=0; i<w; i++) {
	 *p = (int)(a*colorbg.r + (1-a)*(*p));
	 p++;
	 *p = (int)(a*colorbg.g + (1-a)*(*p));
	 p++;
	 *p = (int)(a*colorbg.b + (1-a)*(*p));
	 p++;
      }   
      base += row_w;
   }
}


  
extern "C" 
{
      
   GDKImageFilter *GetImageFilter( int index ) {
	
      switch( index ) {	   
       case 0: return new DVTitler();
      }
	
      return NULL;
   }
}

   
   
