/*
 *  * Copyright (C) 2002 Alejandro Aguilar Sierra <asierra@servidor.unam.mx>
 *  *
 *  * This program is free software; you can redistribute it and/or modify
 *  * it under the terms of the GNU General Public License as published by
 *  * the Free Software Foundation; either version 2 of the License, or
 *  * (at your option) any later version.
 *  *
 *  * This program is distributed in the hope that it will be useful,
 *  * but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  * GNU General Public License for more details.
 *  *
 *  * You should have received a copy of the GNU General Public License
 *  * along with this program; if not, write to the Free Software Foundation,
 *  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *  */


#include "dvtitler.h"
#include "ft_titler.h"


FT_Titler::FT_Titler(int size, char *text, char *fname): size(size), text(text), fname(fname)
{ 
   num_chars = strlen(text);
   if (fname[0]=='-')
     setXFontName(fname);
   initString();   
}


void FT_Titler::drawText(int start_x, int start_y) 
{   
   FT_Error      error;
   FT_Vector pen;
   
   for ( int n = 0; n < num_glyphs; n++ ) {
      FT_Glyph  image;

      image = glyphs[n];

      pen.x = start_x + pos[n].x;
      pen.y = start_y + pos[n].y;

      error = FT_Glyph_To_Bitmap( &image, ft_render_mode_normal,
				  &pen, 0 );
      if (!error) {
	 FT_BitmapGlyph  bit = (FT_BitmapGlyph)image;
	 
	 drawBitmap( bit->bitmap,
		     pen.x + bit->left,
		     pen.y - bit->top );	   
	 FT_Done_Glyph( image );
      }
   }
}

void FT_Titler::drawBitmap(FT_Bitmap &bitmap, FT_Int bitmap_left, FT_Int bitmap_top)
{
   DVColor s;
   float alpha = 0, t;
   uint8_t r, g, b;
   uint8_t *pbase = pixels + bitmap_left*3 + bitmap_top*row_w;
   
   for (int j=0; j < bitmap.rows; j++) {
      unsigned char*  buf = bitmap.buffer + j*bitmap.width;
      //abs(bitmap.pitch);
      uint8_t *p = pbase;
      for (int i=0; i < bitmap.width; i++) {
	 r = *( p );
	 g = *( p + 1 );
	 b = *( p + 2 );	 
	 t = (float)*(buf++)/255.0;
	 s.r =  (uint8_t)((1-t)*colorbg.r + t*colorfg.r);
	 s.g =  (uint8_t)((1-t)*colorbg.g + t*colorfg.g);
	 s.b =  (uint8_t)((1-t)*colorbg.b + t*colorfg.b);
	 alpha = ((1-t)*colorbg.a + t*colorfg.a)/255.0;
	 *p ++ = (uint8_t)(s.r*alpha + (1 - alpha)*r);
	 *p ++ = (uint8_t)(s.g*alpha + (1 - alpha)*g);
	 *p ++ = (uint8_t)(s.b*alpha + (1 - alpha)*b);
      }
      pbase += row_w;
   }
}   


void FT_Titler::getMetrics(int &w, int &h) 
{
   FT_BBox  bbox;
   computeStringbbox(&bbox);
   
   w = bbox.xMax - bbox.xMin;
   h = bbox.yMax - bbox.yMin;  
}


void FT_Titler::initString() 
{  
   FT_Library  library;
   FT_Face      face; 
   FT_GlyphSlot  slot = face->glyph;  // a small shortcut
   FT_UInt       glyph_index;
   FT_Bool       use_kerning;
   FT_UInt       previous;
   FT_Error      error;
   int           pen_x, pen_y, n;

   pen_x = 0;   /* start at (0,0) !! */
   pen_y = 0;

   num_glyphs  = 0;
   previous    = 0;

   error = FT_Init_FreeType( &library );
   if ( error ) {
      fprintf(stderr, "Error initializing FT\n");
      fflush(stderr);
      num_glyphs = 0;
      return;
   }
   
   error = FT_New_Face( library,
                        fname,   // "/usr/X11R6/lib/X11/fonts/TrueType/arial.ttf",
                        0,
                        &face );
   if ( error ) {
      fprintf(stderr, "Error New Face FT\n");
      fflush(stderr);
      num_glyphs = 0;
      return;
   }

   error = FT_Set_Char_Size(
                            face,    /* handle to face object           */
                            0,       /* char_width in 1/64th of points  */
                            size*64,   /* char_height in 1/64th of points */
                            0,     /* horizontal device resolution    */
                            0 );   /* vertical device resolution      */
   if ( error ) {
      fprintf(stderr, "Error Char Size FT\n");
      fflush(stderr);
      num_glyphs = 0;
      return;
   }
   
   use_kerning = FT_HAS_KERNING(face);
   
   for ( n = 0; n < num_chars; n++ ) {
      // convert character code to glyph index
      glyph_index = FT_Get_Char_Index( face, text[n] );

      // retrieve kerning distance and move pen position
      if ( use_kerning && previous && glyph_index ) {
	 FT_Vector  delta;

	 FT_Get_Kerning( face, previous, glyph_index,
			 ft_kerning_default, &delta );

	 pen_x += delta.x >> 6;
      }

      // store current pen position
      pos[ num_glyphs ].x = pen_x;
      pos[ num_glyphs ].y = pen_y;
//      fprintf(stderr, "Pen pos %d %d\n", pen_x, pen_y);
      
      // load glyph image into the slot. DO NOT RENDER IT !!
      error = FT_Load_Glyph( face, glyph_index, FT_LOAD_DEFAULT );
      if (error) continue;  // ignore errors, jump to next glyph

      // extract glyph image and store it in our table
      error = FT_Get_Glyph( face->glyph, & glyphs[num_glyphs] );
      if (error) continue;  // ignore errors, jump to next glyph

      // increment pen position
      pen_x += face->glyph->advance.x >> 6;

      // record current glyph index
      previous = glyph_index;

      // increment number of glyphs
      num_glyphs++;
   }
//   fprintf(stderr, "Glyphs %d\n", num_glyphs);
}


void FT_Titler::computeStringbbox( FT_BBox  *abbox ) {
   FT_BBox  bbox;

   bbox.xMin = bbox.yMin =  32000;
   bbox.xMax = bbox.yMax = -32000;

   for ( int n = 0; n < num_glyphs; n++ ) {
      FT_BBox   glyph_bbox;

      FT_Glyph_Get_CBox( glyphs[n], ft_glyph_bbox_pixels, &glyph_bbox );

      glyph_bbox.xMin += pos[n].x;
      glyph_bbox.xMax += pos[n].x;
      glyph_bbox.yMin += pos[n].y;
      glyph_bbox.yMax += pos[n].y;
      
      if (glyph_bbox.xMin < bbox.xMin)
	bbox.xMin = glyph_bbox.xMin;

      if (glyph_bbox.yMin < bbox.yMin)
	bbox.yMin = glyph_bbox.yMin;

      if (glyph_bbox.xMax > bbox.xMax)
	bbox.xMax = glyph_bbox.xMax;

      if (glyph_bbox.yMax > bbox.yMax)
	bbox.yMax = glyph_bbox.yMax;
   }

   if ( bbox.xMin > bbox.xMax ) {
      bbox.xMin = 0;
      bbox.yMin = 0;
      bbox.xMax = 0;
      bbox.yMax = 0;
   }
   
   *abbox = bbox;
}
   


// UGLY DIRTY HACK TO GET THE PATH AND FILENAME FROM AN X FONT NAME
// It doesn't work with remote font servers or non scalable fonts
// Only works with fonts in a format supported by freetype
#include <string>
#include <X11/Xlib.h>
#include <X11/Xutil.h>

extern Display *gdk_display;

void FT_Titler::setXFontName(char *fontname) 
{
   int xx = 1, nf, l;
   char **fontdirs = XGetFontPath(gdk_display, &nf);
   string subfn, fn = fontname;
   for (int i=0; i < 5; i++)
     {	
	xx = fn.find('-', xx);
	xx++;
     }
   subfn = fn.substr(0, xx);
   xx = fn.find('-', xx);
   l = xx+1;
   l = fn.find('-', l);
   sscanf(fn.substr(xx+1, l).c_str(), "%d", &size);
   fprintf(stderr, "Size %d [%s]{%s}\n", size, 
	   fn.substr(xx+1, l-xx-1).c_str(),
	   subfn.c_str());
   while (--nf > 0) {
      string file = fontdirs[nf];
      file += "/fonts.dir";
      fprintf(stderr, "FDir %s\n", fontdirs[nf]);
      FILE *inf = fopen(file.c_str(), "r");
      if (inf) {	   
	 while (!feof(inf)) {
	    static char s[1024];
	    fgets(&s[0], 1000, inf);
	    string t, tmps = s;
	    int i = tmps.find('-');
	    int l = subfn.size();
	    int n = strncasecmp(subfn.c_str(), &tmps.c_str()[i], l);
	    if (n == 0) {
	       fprintf(stderr, "FILE! [%s] [%s]\n", &s[0], tmps.substr(0, i-1).c_str());	       
	       sprintf(s, "%s/%s", fontdirs[nf], tmps.substr(0, i-1).c_str());	      
	       if (strcmp(".gz", &s[strlen(s)-3])==0) {
		  char cmd[1024];
		  sprintf(cmd, "cp %s /tmp; gunzip /tmp/%s", s, tmps.substr(0, i-1).c_str());
		  sprintf(s, "/tmp/%s", tmps.substr(0, i-1).c_str());
		  system(cmd);
	       }		   
	       fname = &s[0];	       
	       return;
	    }
	 }
      }	 	       
   } 
   fprintf(stderr, "Error: Font file not found\n");
   num_glyphs = 0;
}

